package org.expeditee.items.MagneticConstraint.Actions;

import java.awt.Point;
import java.awt.Rectangle;

import org.expeditee.gui.DisplayIO;
import org.expeditee.gui.FrameIO;
import org.expeditee.items.Item;
import org.expeditee.items.Text;
import org.expeditee.items.MagneticConstraint.MagneticConstraintAction;
import org.expeditee.items.MagneticConstraint.MagneticConstraints;
import org.expeditee.items.MagneticConstraint.Utilities.BoxLogic;
import org.expeditee.items.MagneticConstraint.Utilities.Line;
import org.expeditee.items.MagneticConstraint.Utilities.Paragraph;
import org.expeditee.items.MagneticConstraint.Utilities.TextLogic;

public class BackspaceAction extends MagneticConstraintAction {

//	if(newX < this._x) {
//		final java.util.Map<Thread, StackTraceElement[]> allStackTraces = Thread.getAllStackTraces();
//		for(final Thread thr : allStackTraces.keySet()) {
//			System.err.println(thr.getName());
//			final StackTraceElement[] stackTraceElements = allStackTraces.get(thr);
//			for(final StackTraceElement ln : stackTraceElements) System.err.println(ln);
//			System.err.println();
//		}	
//	}
	@Override
	public boolean exec(final Item item) {
		if(item == null) return false;
		if(item.getMagnetizedItemTop() != -1 && TextLogic.XIsBeforeCharacters((Text) item, DisplayIO.getMouseX())) {
			MagneticConstraints.Log(this.getClass(), new Item[]{item}, 
					Line.getLineContainingToken(item).toArray(new Item[]{}), 
					"Top Neighbor detected (id:" + item.getMagnetizedItemTop() + "). Merging this line with line above.");
			return mergeLines(item);
		} else if(TextLogic.XIsBeforeCharacters((Text) item, DisplayIO.getMouseX())) {
			if(item.getMagnetizedItemLeft() != -1) {
				callback(null);
				final Item toLeft = item.getParent().getItemWithID(item.getMagnetizedItemLeft());
				final int initialDistance = item.getX() + item.getBoundsWidth() - toLeft.getX();
				toLeft.setText(toLeft.getText() + item.getText());
				item.delete();
				final int alteredDistance = toLeft.getBoundsWidth();
				final float delta = (initialDistance - alteredDistance) * 0.8f;
				callback(item);
				new AttractTextAction().exec(toLeft, delta);
				FrameIO.ForceSaveFrame(toLeft.getParent());
				FrameIO.Reload();
				callback(null);
				MagneticConstraints.Log(this.getClass(), new Item[]{item, toLeft}, 
						Line.getLineContainingToken(item).toArray(new Item[]{}), 
						"Left Neighbor detected (id:" + item.getMagnetizedItemLeft() + 
						"). Merging second primary into first.  Calling AttractTextAction.");
				return true;
			} else return false;
		} else return false;
	}

	private boolean mergeLines(final Item item) {
		callback(null);
		//1. Get current and last line
		final Line currentLine = Line.getLineFromToken(item);
		final Line lastLine = currentLine.last();
		//2. Is there a last line?  If not we don't do anything
		if(lastLine == null) return false;
		//3. Is the last line in the same paragraph?
		final Paragraph currentLineParagraph = Paragraph.getParagraphFromLine(currentLine);
		final Paragraph lastLineParagraph = Paragraph.getParagraphFromLine(lastLine);
		if(!lastLineParagraph.containsAll(currentLineParagraph)) {
			//3a. No.  We don't do anything.
			return false;
		}
		//NB: at this point we know we have a last line that is in the same paragraph.  So we have to move tokens about.
		//4. Move the tokens.
		for(final Item token: currentLine) if(!moveToLineAbove(token)) break;
		//5. Move the cursor
		DisplayIO.MoveCursorToEndOfItem(lastLine.getLast());
		
		//6. Move lines below up
		final Line nextLine = currentLine.next();
		if(nextLine != null && currentLineParagraph.contains(nextLine))
			Paragraph.getParagraphFromLine(nextLine).deltaY(-Line.getLineHeight(currentLine));
		//7. Refresh the screen and use callback to update the magnets.
		FrameIO.ForceSaveFrame(item.getParent());
		FrameIO.Reload();
		callback(item);
		callback(null);
		return true;
	}
	
	private boolean moveToLineAbove(final Item token) {
		final Line currentLine = Line.getLineContainingToken(token);
		final Line lineAbove = currentLine.last();
		final Rectangle lastLineBoundingBox = lineAbove.getBoundingBox();
		final Point locationToMoveTo = new Point((int) (lastLineBoundingBox.getX() + lastLineBoundingBox.getWidth() + 10), (int) lastLineBoundingBox.getMinY());
		token.setPosition(locationToMoveTo);
		currentLine.removeFirst();
		currentLine.deltaX(-token.getBoundsWidth());
		callback(token);
		if(BoxLogic.boxCollision(token)) {
			BoxLogic.resolveCollision(token, this.callbackAction);
			callback(token);
			return false;
		}
		return true;
	}

	@Override
	public boolean invert(Item item) {
		System.err.println("Repel text up action does not have a inverted implementation.");
		return false;
	}
}
