package org.expeditee.items.MagneticConstraint.Actions;

import java.awt.Point;

import org.expeditee.gui.DisplayIO;
import org.expeditee.gui.FrameIO;
import org.expeditee.items.*;
import org.expeditee.items.MagneticConstraint.MagneticConstraintAction;
import org.expeditee.items.MagneticConstraint.Utilities.Line;
import org.expeditee.items.MagneticConstraint.Utilities.Paragraph;
import org.expeditee.items.MagneticConstraint.Utilities.TextLogic;

public class RepelTextDownAction extends MagneticConstraintAction {
	
	@Override
	public boolean exec(final Item item) {
		if(item == null) return false;
		//Calculate content that needs to be moved to next line.
			//1. Get remainder of line
		final Line remainderOfLine = Line.getLineFromToken(item);
			//2. Check if the current item should be included; remove it if not.
		if(remainderOfLine.getFirst().getText().isEmpty() || 
				(remainderOfLine.size() > 0 && TextLogic.XIsTowardsRight(DisplayIO.getMouseX(), (Text) remainderOfLine.getFirst())))
			remainderOfLine.removeFirst();
		//Move content below this line down
			//1. Get current line & current paragraph.
		final Line currentLine = Line.getLineContainingToken(item);
		final Paragraph currentParagraph = Paragraph.getParagraphFromLine(currentLine);
			//2. Split current paragraph
		final Point newLineAt = currentParagraph.split(currentLine);
		//Move content contained in 'remainderOfLine'
			//1. Is remainderOfLine not Empty?
		if(!remainderOfLine.isEmpty()) {
				//2a. Yes. Move the content
			final Point first = remainderOfLine.get(0).getPosition();
			for(final Item token : remainderOfLine) {
				double newX = token.getX() - first.getX() + newLineAt.getX();
				double newY = token.getY() - first.getY() + newLineAt.getY();
				token.setPosition((float) newX, (float) newY);
			}
				//2b. Move the cursor; refresh the screen and use callback to update the magnets
			DisplayIO.setCursorPosition(remainderOfLine.get(0).getPosition());
			FrameIO.ForceSaveFrame(item.getParent());
			FrameIO.Reload();
			callback(null);
		} else {
				//3a No. Move the cursor; refresh the screen and use callback to update the magnets
			DisplayIO.setCursorPosition(newLineAt);
			FrameIO.ForceSaveFrame(item.getParent());
			FrameIO.Reload();
			callback(null);
		}
		return true;
	}

	@Override
	public boolean invert(Item item) {
		System.err.println("Repel text down action does not have a inverted implementation.");
		return false;
	}

}
